//C6713dskinit.c Includes functions from TI in the C6713 CSL and C6713DSK BSL

#include "C6713dskinit.h"
#define using_bios;

Uint32 fs=DSK6713_AIC23_FREQ_8KHZ;
//Uint32 fs=DSK6713_AIC23_FREQ_16KHZ;
//Uint32 fs=DSK6713_AIC23_FREQ_24KHZ;
//Uint32 fs=DSK6713_AIC23_FREQ_32KHZ;
//Uint32 fs=DSK6713_AIC23_FREQ_44KHZ;
//Uint32 fs=DSK6713_AIC23_FREQ_48KHZ;
//Uint32 fs=DSK6713_AIC23_FREQ_96KHZ;

union {Uint32 combo; short channel[2];} AIC23_data;


//#define using_bios                  //if BIOS don't use top of vector table
extern Uint32 fs;            			//for sampling frequency

void c6713_dsk_init()       			//dsp-peripheral initialization
{
DSK6713_init();                   	//call BSL to init DSK-EMIF,PLL)

hAIC23_handle=DSK6713_AIC23_openCodec(0, &config);//handle(pointer) to codec
DSK6713_AIC23_setFreq(hAIC23_handle, fs);  //set sample rate
MCBSP_config(DSK6713_AIC23_DATAHANDLE,&AIC23CfgData);//interface 32 bits toAIC23

MCBSP_start(DSK6713_AIC23_DATAHANDLE, MCBSP_XMIT_START | MCBSP_RCV_START |
	MCBSP_SRGR_START | MCBSP_SRGR_FRAMESYNC, 220);//start data channel again
}

void comm_poll()					 		//added for communication/init using polling
{
	poll=1;              				//1 if using polling
   c6713_dsk_init();    				//init DSP and codec
}

void comm_intr()						 	//for communication/init using interrupt
{
	poll=0;                        	//0 since not polling
   IRQ_globalDisable();           	//disable interrupts
	c6713_dsk_init(); 					//init DSP and codec
	CODECEventId=MCBSP_getXmtEventId(DSK6713_AIC23_codecdatahandle);//McBSP1 Xmit

#ifndef using_bios						//do not need to point to vector table
  	IRQ_setVecs(vectors);     			//point to the IRQ vector table
#endif										//since interrupt vector handles this

  	IRQ_map(CODECEventId, 11);			//map McBSP1 Xmit to INT11
  	IRQ_reset(CODECEventId);    		//reset codec INT 11
   IRQ_globalEnable();       			//globally enable interrupts
  	IRQ_nmiEnable();          			//enable NMI interrupt
   IRQ_enable(CODECEventId);			//enable CODEC eventXmit INT11

	output_sample(0);        			//start McBSP interrupt outputting a sample
}

void output_sample(int out_data)    //for out to Left and Right channels
{
	short CHANNEL_data;

	AIC_data.uint=0;                 //clear data structure
	AIC_data.uint=out_data;          //32-bit data -->data structure

//The existing interface defaults to right channel. To default instead to the
//left channel and use output_sample(short), left and right channels are swapped
//In main source program use LEFT 0 and RIGHT 1 (opposite of what is used here)
	CHANNEL_data=AIC_data.channel[RIGHT]; 			//swap left and right channels
	AIC_data.channel[RIGHT]=AIC_data.channel[LEFT];
	AIC_data.channel[LEFT]=CHANNEL_data;
   if (poll) while(!MCBSP_xrdy(DSK6713_AIC23_DATAHANDLE));//if ready to transmit
		MCBSP_write(DSK6713_AIC23_DATAHANDLE,AIC_data.uint);//write/output data
}

void output_left_sample(short out_data)     	  	 //for output from left channel
{
	AIC_data.uint=0;                              //clear data structure
	AIC_data.channel[LEFT]=out_data;   //data from Left channel -->data structure

	if (poll) while(!MCBSP_xrdy(DSK6713_AIC23_DATAHANDLE));//if ready to transmit
		MCBSP_write(DSK6713_AIC23_DATAHANDLE,AIC_data.uint);//output left channel
}

void output_right_sample(short out_data)  		//for output from right channel
{
	AIC_data.uint=0;                             //clear data structure
	AIC_data.channel[RIGHT]=out_data; //data from Right channel -->data structure

	if (poll) while(!MCBSP_xrdy(DSK6713_AIC23_DATAHANDLE));//if ready to transmit
		MCBSP_write(DSK6713_AIC23_DATAHANDLE,AIC_data.uint);//output right channel
}

Uint32 input_sample()                      	  	//for 32-bit input
{
	short CHANNEL_data;

	if (poll) while(!MCBSP_rrdy(DSK6713_AIC23_DATAHANDLE));//if ready to receive
		 AIC_data.uint=MCBSP_read(DSK6713_AIC23_DATAHANDLE);//read data

//Swapping left and right channels (see comments in output_sample())
	CHANNEL_data=AIC_data.channel[RIGHT]; 			//swap left and right channel
	AIC_data.channel[RIGHT]=AIC_data.channel[LEFT];
	AIC_data.channel[LEFT]=CHANNEL_data;

	return(AIC_data.uint);
}

short input_left_sample()                   		//input to left channel
{
	if (poll) while(!MCBSP_rrdy(DSK6713_AIC23_DATAHANDLE));//if ready to receive
	 AIC_data.uint=MCBSP_read(DSK6713_AIC23_DATAHANDLE);//read into left channel
	return(AIC_data.channel[LEFT]);					//return left channel data
}

short input_right_sample()                  		//input to right channel
{
	if (poll) while(!MCBSP_rrdy(DSK6713_AIC23_DATAHANDLE));//if ready to receive
	 AIC_data.uint=MCBSP_read(DSK6713_AIC23_DATAHANDLE);//read into right channel
	return(AIC_data.channel[RIGHT]); 				//return right channel data
}

// DSP coefficients, variables, and constants
// Section 11.5 Program 11.2
int ulawcode;
/* digital mulaw definition*/
// sign-magnitude format: s segment quantization 
// s=1 for the positive value, s =0 for the negative value
// segment defines compression
// quantization with 16 levels
// see Section 11.2, Chapter 11
/* segment      12-bit PCM      4-bit quantization interval    */
/*    0	        s0000000ABCD      s000ABCD                     */
/*    1	        s0000001ABCD      s001ABCD                     */
/*    2	        s000001ABCDX      s010ABCD                     */
/*    3	        s00001ABCDXX      s011ABCD                     */
/*    4	        s0001ABCDXXX      s100ABCD                     */
/*    5	        s001ABCDXXXX      s101ABCD                     */
/*    6	        s01ABCDXXXXX      s110ABCD                     */
/*    7	        s1ABCDXXXXXX      s111ABCD                     */
//
/* segment      recovered 12-bit PCM         */
/*    0	        s0000000ABCD                         */
/*    1	        s0000001ABCD                         */
/*    2	        s000001ABCD1                         */
/*    3	        s00001ABCD10                         */
/*    4	        s0001ABCD100                         */
/*    5	        s001ABCD1000                         */
/*    6	        s01ABCDq0000                         */
/*    7	        s1ABCD100000                         */
/* note that, convert the two's complement form to the sign-magnitude form    */
/* before quantizing and at decoder site, convert                           */
/* the sign-magnitude form to the two's complement form for DAC  */

interrupt void c_int11()
{
   int lc;
   int rc;
   int lcnew;
   int rcnew;
   int tmp, dec;
//Left channel and right channel are switched
//  if a stereo cable is used,
//  tip=right channel; midddle ring=left channel
//  sleeve (bottom)=gounrd; 
	AIC23_data.combo=input_sample();
	lc=(float) (AIC23_data.channel[LEFT]);
	rc= (float) (AIC23_data.channel[RIGHT]);
	// Insert DSP algorithm bellow 
  /* encoder :*/
  rcnew=rc; // avoid warning
  tmp =lc;
  if (tmp <0 )
  { tmp=-tmp;  // get magnitude bits to work with
  }
  tmp =(tmp>>4); // linear scale down to 12 bits to use the u-255 law table
   if( (tmp&0x07f0)==0x0)   // segment 0
   {
        ulawcode= (tmp&0x000f);
   }
   if( (tmp&0x07f0)==0x0010)  // segment 1
   {
        ulawcode= (tmp&0x00f);
        ulawcode= ulawcode | 0x10;
   }
   if( (tmp&0x07E0)==0x0020)   // segment 2
   {
        ulawcode= (tmp&0x001f)>>1;
        ulawcode = ulawcode |0x20;
   }
   if( (tmp&0x07c0)==0x0040)   // segment 3
   {
        ulawcode= (tmp&0x003f)>>2;
        ulawcode= ulawcode | 0x30;
   }
   if( (tmp&0x0780)==0x0080) // segment 4
   {
        ulawcode= (tmp&0x007f)>>3;
        ulawcode =ulawcode | 0x40;
   }
   if( (tmp&0x0700)==0x0100) // segment 5
   {
        ulawcode= (tmp&0x00ff)>>4;
        ulawcode = ulawcode | 0x50;
   }
   if( (tmp&0x0600)==0x0200)  // segment 6
   {
        ulawcode= (tmp&0x01ff)>>5;
        ulawcode = ulawcode | 0x60;
   }
   if( (tmp&0x0400)==0x0400) // segment 7
   {
        ulawcode= (tmp&0x03ff)>>6;
        ulawcode=ulawcode | 0x70;
   }
    if (lc>=0)
    {
        ulawcode= ulawcode|0x80;
    }
   /* ulawcode (8 bit) */
   /* Decoder: */
       tmp = ulawcode&0x7f;
       tmp = (tmp>>4);
   if ( tmp == 0x0) // segment 0
   {
        dec = ulawcode&0xf;   
   }
   if ( tmp == 0x1)  // segment 1
   {
        dec = ulawcode&0xf | 0x10;
   }  
   if ( tmp == 0x2)   // segment 2
   {
        dec = ((ulawcode&0xf)<<1) | 0x20;
        dec= dec |0x01;
   }
   if ( tmp == 0x3)   // segment 3
   {
        dec = ((ulawcode&0xf)<<2) | 0x40;
        dec = dec|0x02;
   }  
   if ( tmp == 0x4)   // segment 4
   {
        dec = ((ulawcode&0xf)<<3) | 0x80;
        dec = dec|0x04;
   }
   if ( tmp == 0x5)   // segment 5
   {
        dec = ((ulawcode&0xf)<<4) | 0x0100;
        dec = dec|0x08;
   }
   if ( tmp == 0x6)   // segment 6
   {
        dec = ((ulawcode&0xf)<<5) | 0x0200;
        dec =dec|0x10;
   }
   if ( tmp == 0x7)   // segment 7
   {
        dec = ((ulawcode&0xf)<<6) | 0x0400;
        dec = dec |0x20;
   }
     tmp =ulawcode & 0x80; 
   lc =dec;
   if (tmp == 0x00)
   {
      lc =-dec;   // back to 2's complement form
   }
    lc = (lc<<4);  // linear scale up to 16 bits 
// End of DSP algorithm
     lcnew=lc; // Send DAC
	 rcnew=lc; // Send DAC
	//
	AIC23_data.channel[LEFT]=(short) lcnew;
    AIC23_data.channel[RIGHT]=(short) rcnew;
	output_sample(AIC23_data.combo);
}



void main()
{
  	comm_intr();
	while(1);
}






