//C6713dskinit.c Includes functions from TI in the C6713 CSL and C6713DSK BSL

#include "C6713dskinit.h"
#define using_bios;

Uint32 fs=DSK6713_AIC23_FREQ_8KHZ;
//Uint32 fs=DSK6713_AIC23_FREQ_16KHZ;
//Uint32 fs=DSK6713_AIC23_FREQ_24KHZ;
//Uint32 fs=DSK6713_AIC23_FREQ_32KHZ;
//Uint32 fs=DSK6713_AIC23_FREQ_44KHZ;
//Uint32 fs=DSK6713_AIC23_FREQ_48KHZ;
//Uint32 fs=DSK6713_AIC23_FREQ_96KHZ;

union {Uint32 combo; short channel[2];} AIC23_data;


//#define using_bios                  //if BIOS don't use top of vector table
extern Uint32 fs;            			//for sampling frequency

void c6713_dsk_init()       			//dsp-peripheral initialization
{
DSK6713_init();                   	//call BSL to init DSK-EMIF,PLL)

hAIC23_handle=DSK6713_AIC23_openCodec(0, &config);//handle(pointer) to codec
DSK6713_AIC23_setFreq(hAIC23_handle, fs);  //set sample rate
MCBSP_config(DSK6713_AIC23_DATAHANDLE,&AIC23CfgData);//interface 32 bits toAIC23

MCBSP_start(DSK6713_AIC23_DATAHANDLE, MCBSP_XMIT_START | MCBSP_RCV_START |
	MCBSP_SRGR_START | MCBSP_SRGR_FRAMESYNC, 220);//start data channel again
}

void comm_poll()					 		//added for communication/init using polling
{
	poll=1;              				//1 if using polling
   c6713_dsk_init();    				//init DSP and codec
}

void comm_intr()						 	//for communication/init using interrupt
{
	poll=0;                        	//0 since not polling
   IRQ_globalDisable();           	//disable interrupts
	c6713_dsk_init(); 					//init DSP and codec
	CODECEventId=MCBSP_getXmtEventId(DSK6713_AIC23_codecdatahandle);//McBSP1 Xmit

#ifndef using_bios						//do not need to point to vector table
  	IRQ_setVecs(vectors);     			//point to the IRQ vector table
#endif										//since interrupt vector handles this

  	IRQ_map(CODECEventId, 11);			//map McBSP1 Xmit to INT11
  	IRQ_reset(CODECEventId);    		//reset codec INT 11
   IRQ_globalEnable();       			//globally enable interrupts
  	IRQ_nmiEnable();          			//enable NMI interrupt
   IRQ_enable(CODECEventId);			//enable CODEC eventXmit INT11

	output_sample(0);        			//start McBSP interrupt outputting a sample
}

void output_sample(int out_data)    //for out to Left and Right channels
{
	short CHANNEL_data;

	AIC_data.uint=0;                 //clear data structure
	AIC_data.uint=out_data;          //32-bit data -->data structure

//The existing interface defaults to right channel. To default instead to the
//left channel and use output_sample(short), left and right channels are swapped
//In main source program use LEFT 0 and RIGHT 1 (opposite of what is used here)
	CHANNEL_data=AIC_data.channel[RIGHT]; 			//swap left and right channels
	AIC_data.channel[RIGHT]=AIC_data.channel[LEFT];
	AIC_data.channel[LEFT]=CHANNEL_data;
   if (poll) while(!MCBSP_xrdy(DSK6713_AIC23_DATAHANDLE));//if ready to transmit
		MCBSP_write(DSK6713_AIC23_DATAHANDLE,AIC_data.uint);//write/output data
}

void output_left_sample(short out_data)     	  	 //for output from left channel
{
	AIC_data.uint=0;                              //clear data structure
	AIC_data.channel[LEFT]=out_data;   //data from Left channel -->data structure

	if (poll) while(!MCBSP_xrdy(DSK6713_AIC23_DATAHANDLE));//if ready to transmit
		MCBSP_write(DSK6713_AIC23_DATAHANDLE,AIC_data.uint);//output left channel
}

void output_right_sample(short out_data)  		//for output from right channel
{
	AIC_data.uint=0;                             //clear data structure
	AIC_data.channel[RIGHT]=out_data; //data from Right channel -->data structure

	if (poll) while(!MCBSP_xrdy(DSK6713_AIC23_DATAHANDLE));//if ready to transmit
		MCBSP_write(DSK6713_AIC23_DATAHANDLE,AIC_data.uint);//output right channel
}

Uint32 input_sample()                      	  	//for 32-bit input
{
	short CHANNEL_data;

	if (poll) while(!MCBSP_rrdy(DSK6713_AIC23_DATAHANDLE));//if ready to receive
		 AIC_data.uint=MCBSP_read(DSK6713_AIC23_DATAHANDLE);//read data

//Swapping left and right channels (see comments in output_sample())
	CHANNEL_data=AIC_data.channel[RIGHT]; 			//swap left and right channel
	AIC_data.channel[RIGHT]=AIC_data.channel[LEFT];
	AIC_data.channel[LEFT]=CHANNEL_data;

	return(AIC_data.uint);
}

short input_left_sample()                   		//input to left channel
{
	if (poll) while(!MCBSP_rrdy(DSK6713_AIC23_DATAHANDLE));//if ready to receive
	 AIC_data.uint=MCBSP_read(DSK6713_AIC23_DATAHANDLE);//read into left channel
	return(AIC_data.channel[LEFT]);					//return left channel data
}

short input_right_sample()                  		//input to right channel
{
	if (poll) while(!MCBSP_rrdy(DSK6713_AIC23_DATAHANDLE));//if ready to receive
	 AIC_data.uint=MCBSP_read(DSK6713_AIC23_DATAHANDLE);//read into right channel
	return(AIC_data.channel[RIGHT]); 				//return right channel data
}

// DSP coefficients, variables, and constants
/* Real time simulation for noise cancellation using FIR adaptive filter, Section 10.3 */
/* ADC is used as the reference noise channel xn[n] */
/* The desired signal s[n] is a 500 Hz tone generated by the following filter*/
 /* H(z)=0.382683432*z-1/(1-1.847759065*z-1+z-2 */
 /* driven by an impulse seqence x */
/* the corrupting noise 0.5xn(n-5) */
 /* the noisy signal d(n)=s(n)+0.5*xn(n-5) */
float x[2]={5000.0,0.0}; // impulse input
float s[3]={0.0,0.0,0.0}; // signal
float d[1]={0.0}; // corrupted signal
float y[1]={0.0}; // ADF output
float w[16]={0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0};  // ADF coefficients
float xn[16]={0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0};  // reference input buffer
float mu =0.0000000000582; /* most safety convergenece factor */

interrupt void c_int11()
{
   float lc;
   float rc;
   float lcnew;
   float rcnew;
   int i;
   float error;
//Left channel and right channel are switched
//  if a stereo cable is used,
//  tip=right channel; midddle ring=left channel
//  sleeve (bottom)=gounrd; 
	AIC23_data.combo=input_sample();
	lc=(float) (AIC23_data.channel[LEFT]);
	rc= (float) (AIC23_data.channel[RIGHT]);
	// Insert DSP algorithm bellow 
	// only code for one channel
   rcnew=rc; // get rid of warning;
   /* sine wave genearator */
   s[0]=0.382683432*x[1]+1.847759065*s[1]-s[2];
   x[1]=x[0];
   x[0]=0;
   s[2]=s[1];
   s[1]=s[0];
   for(i=15; i>0; i--)  /* buffer update for FIR ADF and noise channel*/
   {
         xn[i]=xn[i-1];
   }
   xn[0]=lc;
   d[0]=s[0]+0.5*xn[5]; // corrupted signal
   /* FIR adaptive filter:*/
   y[0]=0.0;
   for(i=0; i<16; i++)
   {
      y[0] = y[0] +w[i]*xn[i];
   }
   error=d[0]-y[0];
   for (i=0; i<16; i++)
   {
     w[i]=w[i]+2*mu*error*xn[i];
   }
     lcnew= error; //clean signal d[0]-y[0]  
     rcnew= error;
	//
	AIC23_data.channel[LEFT]=(short) lcnew;
    AIC23_data.channel[RIGHT]=(short) rcnew;
	output_sample(AIC23_data.combo);
}



void main()
{
  	comm_intr();
	while(1);
}






