//C6713dskinit.c Includes functions from TI in the C6713 CSL and C6713DSK BSL

#include "C6713dskinit.h"
#define using_bios;
#include "t400.h"  // data file contains 1600 samples
// one second of a 400 Hz sine wave sampled at 1600 Hz 

Uint32 fs=DSK6713_AIC23_FREQ_8KHZ;
//Uint32 fs=DSK6713_AIC23_FREQ_16KHZ;
//Uint32 fs=DSK6713_AIC23_FREQ_24KHZ;
//Uint32 fs=DSK6713_AIC23_FREQ_32KHZ;
//Uint32 fs=DSK6713_AIC23_FREQ_44KHZ;
//Uint32 fs=DSK6713_AIC23_FREQ_48KHZ;
//Uint32 fs=DSK6713_AIC23_FREQ_96KHZ;

union {Uint32 combo; short channel[2];} AIC23_data;


//#define using_bios                  //if BIOS don't use top of vector table
extern Uint32 fs;            			//for sampling frequency

void c6713_dsk_init()       			//dsp-peripheral initialization
{
DSK6713_init();                   	//call BSL to init DSK-EMIF,PLL)

hAIC23_handle=DSK6713_AIC23_openCodec(0, &config);//handle(pointer) to codec
DSK6713_AIC23_setFreq(hAIC23_handle, fs);  //set sample rate
MCBSP_config(DSK6713_AIC23_DATAHANDLE,&AIC23CfgData);//interface 32 bits toAIC23

MCBSP_start(DSK6713_AIC23_DATAHANDLE, MCBSP_XMIT_START | MCBSP_RCV_START |
	MCBSP_SRGR_START | MCBSP_SRGR_FRAMESYNC, 220);//start data channel again
}

void comm_poll()					 		//added for communication/init using polling
{
	poll=1;              				//1 if using polling
   c6713_dsk_init();    				//init DSP and codec
}

void comm_intr()						 	//for communication/init using interrupt
{
	poll=0;                        	//0 since not polling
   IRQ_globalDisable();           	//disable interrupts
	c6713_dsk_init(); 					//init DSP and codec
	CODECEventId=MCBSP_getXmtEventId(DSK6713_AIC23_codecdatahandle);//McBSP1 Xmit

#ifndef using_bios						//do not need to point to vector table
  	IRQ_setVecs(vectors);     			//point to the IRQ vector table
#endif										//since interrupt vector handles this

  	IRQ_map(CODECEventId, 11);			//map McBSP1 Xmit to INT11
  	IRQ_reset(CODECEventId);    		//reset codec INT 11
   IRQ_globalEnable();       			//globally enable interrupts
  	IRQ_nmiEnable();          			//enable NMI interrupt
   IRQ_enable(CODECEventId);			//enable CODEC eventXmit INT11

	output_sample(0);        			//start McBSP interrupt outputting a sample
}

void output_sample(int out_data)    //for out to Left and Right channels
{
	short CHANNEL_data;

	AIC_data.uint=0;                 //clear data structure
	AIC_data.uint=out_data;          //32-bit data -->data structure

//The existing interface defaults to right channel. To default instead to the
//left channel and use output_sample(short), left and right channels are swapped
//In main source program use LEFT 0 and RIGHT 1 (opposite of what is used here)
	CHANNEL_data=AIC_data.channel[RIGHT]; 			//swap left and right channels
	AIC_data.channel[RIGHT]=AIC_data.channel[LEFT];
	AIC_data.channel[LEFT]=CHANNEL_data;
   if (poll) while(!MCBSP_xrdy(DSK6713_AIC23_DATAHANDLE));//if ready to transmit
		MCBSP_write(DSK6713_AIC23_DATAHANDLE,AIC_data.uint);//write/output data
}

void output_left_sample(short out_data)     	  	 //for output from left channel
{
	AIC_data.uint=0;                              //clear data structure
	AIC_data.channel[LEFT]=out_data;   //data from Left channel -->data structure

	if (poll) while(!MCBSP_xrdy(DSK6713_AIC23_DATAHANDLE));//if ready to transmit
		MCBSP_write(DSK6713_AIC23_DATAHANDLE,AIC_data.uint);//output left channel
}

void output_right_sample(short out_data)  		//for output from right channel
{
	AIC_data.uint=0;                             //clear data structure
	AIC_data.channel[RIGHT]=out_data; //data from Right channel -->data structure

	if (poll) while(!MCBSP_xrdy(DSK6713_AIC23_DATAHANDLE));//if ready to transmit
		MCBSP_write(DSK6713_AIC23_DATAHANDLE,AIC_data.uint);//output right channel
}

Uint32 input_sample()                      	  	//for 32-bit input
{
	short CHANNEL_data;

	if (poll) while(!MCBSP_rrdy(DSK6713_AIC23_DATAHANDLE));//if ready to receive
		 AIC_data.uint=MCBSP_read(DSK6713_AIC23_DATAHANDLE);//read data

//Swapping left and right channels (see comments in output_sample())
	CHANNEL_data=AIC_data.channel[RIGHT]; 			//swap left and right channel
	AIC_data.channel[RIGHT]=AIC_data.channel[LEFT];
	AIC_data.channel[LEFT]=CHANNEL_data;

	return(AIC_data.uint);
}

short input_left_sample()                   		//input to left channel
{
	if (poll) while(!MCBSP_rrdy(DSK6713_AIC23_DATAHANDLE));//if ready to receive
	 AIC_data.uint=MCBSP_read(DSK6713_AIC23_DATAHANDLE);//read into left channel
	return(AIC_data.channel[LEFT]);					//return left channel data
}

short input_right_sample()                  		//input to right channel
{
	if (poll) while(!MCBSP_rrdy(DSK6713_AIC23_DATAHANDLE));//if ready to receive
	 AIC_data.uint=MCBSP_read(DSK6713_AIC23_DATAHANDLE);//read into right channel
	return(AIC_data.channel[RIGHT]); 				//return right channel data
}

// DSP coefficients, variables, and constants
// Section 12.6 Program 12.10
// data file (t400.h) contains 1600 samples
// one second of a 400 Hz sine wave sampled at 1600 Hz
/* up sampling scheme, see Section 12.1.2, in Chapter 12 */
/* The implementation increases the sampling rate of 1.6 kHz to 8 kHz */
/* and the interpolated data at the 8 kHz */
/* up sampling factor =5, signal passband frequency range 0-400 Hz  */
/* stopband from 800 Hz to 4000 Hz with attenuation of 45 dB */
/* to remove image replicas */                                          
                                                          

/* FIR filter coefficients: FIR lowpass anti-image filter           */
/* with a cut of frequency of 600 Hz, 67 taps, Hamming window for design */


float b[67] ={
   0.0001, 0.0005, 0.0008, 0.0010, 0.0011, 0.0009, 0.0003, -0.0007,
   -0.0019, -0.0030, -0.0037, -0.0036, -0.0024, 0.0000, 0.0033, 0.0068,
   0.0096, 0.0106, 0.0091, 0.0046, -0.0026, -0.0114, -0.0199, -0.0257,
   -0.0265, -0.0204, -0.0064, 0.0152, 0.0427, 0.0732, 0.1028, 0.1277,
   0.1442, 0.1500, 0.1442, 0.1277, 0.1028, 0.0732, 0.0427, 0.0152, 
   -0.0064, -0.0204, -0.0265, -0.0257, -0.0199, -0.0114, -0.0026, 0.0046,
   0.0091, 0.0106, 0.0096, 0.0068, 0.0033, 0.0000, -0.0024, -0.0036,
   -0.0037, -0.0030, -0.0019, -0.0007, 0.0003, 0.0009, 0.0011, 0.0010,
   0.0008, 0.0005, 0.0001}; // FIR filter coefficients

/* input buffer */
float x[67]={0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
float y[1]={0.0};
int L=5; //upsampling factor
int Lcount=0; //count for upsampling;
int cnt=0; // the number of input samples


interrupt void c_int11()
{
   float lc;
   float rc;
   float lcnew;
   float rcnew;
   int i;
//Left channel and right channel are switched
//  if a stereo cable is used,
//  tip=right channel; midddle ring=left channel
//  sleeve (bottom)=gounrd; 
	AIC23_data.combo=input_sample();
	lc=(float) (AIC23_data.channel[LEFT]);
	rc= (float) (AIC23_data.channel[RIGHT]);
	// Insert DSP algorithm bellow 
	lcnew=lc;rcnew=rc;// void warnings
   Lcount++;
   for(i=66; i>0; i--)   // update input buffer with zeros
   {
         x[i]=x[i-1];
   }
   x[0]=0;
   if (Lcount==L)
   {
     x[0]=(float) xin[cnt];  // load new sample for every L samples
     cnt++;
     Lcount =0;
   }
   if (cnt==NUM_DATA) //check the data in the buffer
   {
      cnt=0;
   }
   y[0]=0.0;
   for(i=0;i<67;i++)     // FIR filtering
   {
     y[0]=y[0]+x[i]*b[i];
   }
   y[0] = ((float) L)* y[0]; // scale up by a factor L
   lcnew = y[0];    // sent the up sampled data to DAC    
   rcnew = y[0];
	//
	AIC23_data.channel[LEFT]=(short) lcnew;
    AIC23_data.channel[RIGHT]=(short) rcnew;
	output_sample(AIC23_data.combo);
}



void main()
{
  	comm_intr();
	while(1);
}






